@@ -1,7 +1,233 @@
 package Beam::Emitter;
-our $VERSION = '1.005';
+our $VERSION = '1.006';
 # ABSTRACT: Role for event emitting classes
 
+#pod =head1 SYNOPSIS
+#pod
+#pod     # A simple custom event class to perform data validation
+#pod     { package My::Event;
+#pod         use Moo;
+#pod         extends 'Beam::Event';
+#pod         has data => ( is => 'ro' );
+#pod     }
+#pod
+#pod     # A class that reads and writes data, allowing event handlers to
+#pod     # process the data
+#pod     { package My::Emitter;
+#pod         use Moo;
+#pod         with 'Beam::Emitter';
+#pod
+#pod         sub write_data {
+#pod             my ( $self, @data ) = @_;
+#pod
+#pod             # Give event listeners a chance to perform further processing of
+#pod             # data
+#pod             my $event = $self->emit( "process_data",
+#pod                 class => 'My::Event',
+#pod                 data => \@data,
+#pod             );
+#pod
+#pod             # Give event listeners a chance to stop the write
+#pod             return if $event->is_default_stopped;
+#pod
+#pod             # Write the data
+#pod             open my $file, '>', 'output';
+#pod             print { $file } @data;
+#pod             close $file;
+#pod
+#pod             # Notify listeners we're done writing and send them the data
+#pod             # we wrote
+#pod             $self->emit( 'after_write', class => 'My::Event', data => \@data );
+#pod         }
+#pod     }
+#pod
+#pod     # An event handler that increments every input value in our data
+#pod     sub increment {
+#pod         my ( $event ) = @_;
+#pod         my $data = $event->data;
+#pod         $_++ for @$data;
+#pod     }
+#pod
+#pod     # An event handler that performs data validation and stops the
+#pod     # processing if invalid
+#pod     sub prevent_negative {
+#pod         my ( $event ) = @_;
+#pod         my $data = $event->data;
+#pod         $event->prevent_default if grep { $_ < 0 } @$data;
+#pod     }
+#pod
+#pod     # An event handler that logs the data to STDERR after we've written in
+#pod     sub log_data {
+#pod         my ( $event ) = @_;
+#pod         my $data = $event->data;
+#pod         print STDERR "Wrote data: " . join( ',', @$data );
+#pod     }
+#pod
+#pod     # Wire up our event handlers to a new processing object
+#pod     my $processor = My::Emitter->new;
+#pod     $processor->on( process_data => \&increment );
+#pod     $processor->on( process_data => \&prevent_negative );
+#pod     $processor->on( after_write => \&log_data );
+#pod
+#pod     # Process some data
+#pod     $processor->process_data( 1, 2, 3, 4, 5 );
+#pod     $processor->process_data( 1, 3, 7, -9, 11 );
+#pod
+#pod     # Log data before and after writing
+#pod     my $processor = My::Emitter->new;
+#pod     $processor->on( process_data => \&log_data );
+#pod     $processor->on( after_write => \&log_data );
+#pod
+#pod =head1 DESCRIPTION
+#pod
+#pod This role is used by classes that want to add callback hooks to allow
+#pod users to add new behaviors to their objects. These hooks are called
+#pod "events". A subscriber registers a callback for an event using the
+#pod L</subscribe> or L</on> methods. Then, the class can call those
+#pod callbacks by L<emitting an event with the emit() method|/emit>.
+#pod
+#pod Using the L<Beam::Event> class, subscribers can stop an event from being
+#pod processed, or prevent the default action from happening.
+#pod
+#pod =head2 Using Beam::Event
+#pod
+#pod L<Beam::Event> is an event object with some simple methods to allow subscribers
+#pod to influence the handling of the event. By calling L<the stop
+#pod method|Beam::Event/stop>, subscribers can stop all futher handling of the
+#pod event. By calling the L<the stop_default method|Beam::Event/stop_default>,
+#pod subscribers can allow other subscribers to be notified about the event, but let
+#pod the emitter know that it shouldn't continue with what it was going to do.
+#pod
+#pod For example, let's build a door that notifies when someone tries to open it.
+#pod Different instances of a door should allow different checks before the door
+#pod opens, so we'll emit an event before we decide to open.
+#pod
+#pod     package Door;
+#pod     use Moo;
+#pod     with 'Beam::Emitter';
+#pod
+#pod     sub open {
+#pod         my ( $self, $who ) = @_;
+#pod         my $event = $self->emit( 'before_open' );
+#pod         return if $event->is_default_stopped;
+#pod         $self->open_the_door;
+#pod     }
+#pod
+#pod     package main;
+#pod     my $door = Door->new;
+#pod     $door->open;
+#pod
+#pod Currently, our door will open for anybody. But let's build a door that only
+#pod open opens after noon (to keep us from having to wake up in the morning).
+#pod
+#pod     use Time::Piece;
+#pod     my $restful_door = Door->new;
+#pod
+#pod     $restful_door->on( before_open => sub {
+#pod         my ( $event ) = @_;
+#pod
+#pod         my $time = Time::Piece->now;
+#pod         if ( $time->hour < 12 ) {
+#pod             $event->stop_default;
+#pod         }
+#pod
+#pod     } );
+#pod
+#pod     $restful_door->open;
+#pod
+#pod By calling L<stop_default|Beam::Event/stop_default>, we set the
+#pod L<is_default_stopped|Beam::Event/is_default_stopped> flag, which the door sees
+#pod and decides not to open.
+#pod
+#pod =head2 Using Custom Events
+#pod
+#pod The default C<Beam::Event> is really only useful for notifications. If you want
+#pod to give your subscribers some data, you need to create a custom event class.
+#pod This allows you to add attributes and methods to your events (with all
+#pod the type constraints and coersions you want).
+#pod
+#pod Let's build a door that can keep certain people out. Right now, our door
+#pod doesn't care who is trying to open it, and our subscribers do not get enough
+#pod information to deny entry to certain people.
+#pod
+#pod So first we need to build an event object that can let our subscribers know
+#pod who is knocking on the door.
+#pod
+#pod     package Door::Knock;
+#pod     use Moo;
+#pod     extends 'Beam::Event';
+#pod
+#pod     has who => (
+#pod         is => 'ro',
+#pod         required => 1,
+#pod     );
+#pod
+#pod Now that we can represent who is knocking, let's notify our subscribers.
+#pod
+#pod     package Door;
+#pod     use Moo;
+#pod     use Door::Knock; # Our emitter must load the class, Beam::Emitter will not
+#pod     with 'Beam::Emitter';
+#pod
+#pod     sub open {
+#pod         my ( $self, $who ) = @_;
+#pod         my $event = $self->emit( 'before_open', class => 'Door::Knock', who => $who );
+#pod         return if $event->is_default_stopped;
+#pod         $self->open_the_door;
+#pod     }
+#pod
+#pod Finally, let's build a listener that knows who is allowed in the door.
+#pod
+#pod     my $private_door = Door->new;
+#pod     $private_door->on( before_open => sub {
+#pod         my ( $event ) = @_;
+#pod
+#pod         if ( $event->who ne 'preaction' ) {
+#pod             $event->stop_default;
+#pod         }
+#pod
+#pod     } );
+#pod
+#pod     $private_door->open;
+#pod
+#pod =head2 Without Beam::Event
+#pod
+#pod Although checking C<is_default_stopped> is completely optional, if you do not
+#pod wish to use the C<Beam::Event> object, you can instead call L<emit_args>
+#pod instead of L<emit> to give arbitrary arguments to your listeners.
+#pod
+#pod     package Door;
+#pod     use Moo;
+#pod     with 'Beam::Emitter';
+#pod
+#pod     sub open {
+#pod         my ( $self, $who ) = @_;
+#pod         $self->emit_args( 'open', $who );
+#pod         $self->open_the_door;
+#pod     }
+#pod
+#pod There's no way to stop the door being opened, but you can at least notify
+#pod someone before it does.
+#pod
+#pod =head1 SEE ALSO
+#pod
+#pod =over 4
+#pod
+#pod =item L<Beam::Event>
+#pod
+#pod =item L<Beam::Emitter::Cookbook>
+#pod
+#pod This document contains some useful patterns for your event emitters and
+#pod listeners.
+#pod
+#pod =item L<http://perladvent.org/2013/2013-12-16.html>
+#pod
+#pod Coordinating Christmas Dinner with Beam::Emitter by Yanick Champoux.
+#pod
+#pod =back
+#pod
+#pod =cut
+
 use strict;
 use warnings;
 
@@ -166,13 +392,13 @@ sub un { shift->unsubscribe( @_ ) }
 sub emit {
     my ( $self, $name, %args ) = @_;
 
-    return unless exists $self->_listeners->{$name};
-
     my $class = delete $args{ class } || "Beam::Event";
     $args{ emitter  } ||= $self;
     $args{ name     } ||= $name;
     my $event = $class->new( %args );
 
+    return $event unless exists $self->_listeners->{$name};
+
     # don't use $self->_listeners->{$name} directly, as callbacks may unsubscribe
     # from $name, changing the array, and confusing the for loop
     my @listeners = @{ $self->_listeners->{$name} };
@@ -237,7 +463,7 @@ Beam::Emitter - Role for event emitting classes
 
 =head1 VERSION
 
-version 1.005
+version 1.006
 
 =head1 SYNOPSIS
 
